/*
Kurzbeschreibung
Mit diesem Skript können die Anrufbeantworterdaten der Anrufer aus der Fritzbox ausgelesen werden. Die Informationen werden in einer JSON Struktur zur Verfügung gestellt.
Vorhande Einträge (Indexnummern) auf dem Anrufbeantworter werden ebenfalls in einer JSON Struktur zur verfügung gestellt werden und einzelne Nachrichten auf dem
Anrufbeantworter können auf Basis der Indexnummer gelöscht werden. 
{1}
Einschränkungen:
           - aktuell keine
{1}
{1}
2020-09-03: Initiale Version um Informationen aus der Fritzbox über die API auszulesen
           https://avm.de/service/schnittstellen/?spm=a2c6h.14275010.0.0.202628cfwq7844
           https://forum.iobroker.net/topic/14288/tr-064-fritzbox-anrufbeantworter/21
           https://developer.aliyun.com/mirror/npm/package/iobroker.tr-064
{1}
{1}
2020-09-09: Mit der Funktion Fritzbox_Anrufbeantworter_GetMessageList() werden die auf dem Fritzbox hinterlegten Anrufe auf dem Anrufbeantworter ausgelesen. 
           Getestet mit Fritzbox 7530 SW 7.20, Fritzbox 6590 SW 7.20
{1}
2020-09-12:
           - Konstante NewIndex_Anrufbeantworter eingeführt welche die ID des Anrufbeatworters in der Fritzbox entspricht
           - Mit dem Skript aus https://forum.iobroker.net/topic/15533/tr-64-erkennen-ob-ein-aufruf-auf-ab-gesprochen-wurde/5 wird ermittelt ob der Anrufer
             auf den Anrufbeantworter gesprochen hat und es werden die Daten aus der Fritzbox ausgelesen
           - Datenpunkt (Button) eingefügt um manuell das Auslesen der Daten aus dem Anrufbeantworter zu triggern
           - Im Datenpunkt DP_Fritzbox_AnrufbeantworterDeleteMessage wird der Index der Nachricht auf dem Anrufbeantworter eingetragen welcher gelöscht werden soll
{1}
2020-09-13:
           - Im Datenpunkt DP_Fritzbox_AnrufbeantworterIndexMessage_json wird ein JSON String für das Widget "materialdesign - Select" erzeugt, welches die Index Einträge
             der Anrufereinträge auf dem Anrufbeantworter enthält  
             
2020-09-22:
           - Wenn keine Nachricht auf dem AB vorhanden ist, wird in den Datenpunkt DP_Fritzbox_AnrufbeantworterIndexMessage_json ein Eintrag hinzugefügt,
             dass keine Nachricht vorhanden ist.               
*/
 
const debug = false;
 
const NewIndex_Anrufbeantworter = 0; //ID des Anrufbeantworters in der Fritzbox. Der erste Anrufbeantworter hat die ID 0

const DP_Fritzbox_AnrufbeantworterDaten_json                = "0_userdata.0.FritzBox.AB.Fritzbox_AnrufbeantworterDaten_json";
const DP_Fritzbox_AnrufbeantworterDatenAktualisieren        = "0_userdata.0.FritzBox.AB.Fritzbox_AnrufbeantworterDatenAktualisieren";
const DP_Fritzbox_AnrufbeantworterGesamtAnzahlNachrichten   = "0_userdata.0.FritzBox.AB.Fritzbox_AnrufbeantworterGesamtAnzahlNachrichten";
const DP_Fritzbox_AnrufbeantworterAnzahlNeueNachrichten     = "0_userdata.0.FritzBox.AB.Fritzbox_AnrufbeantworterAnzahlNeueNachrichten"
const DP_Fritzbox_AnrufbeantworterDeleteMessage             = "0_userdata.0.FritzBox.AB.Fritzbox_AnrufbeantworterDeleteMessage";
const DP_Fritzbox_AnrufbeantworterIndexMessage_json         = "0_userdata.0.FritzBox.AB.Fritzbox_AnrufbeantworterIndexMessage_json";

createState(DP_Fritzbox_AnrufbeantworterDaten_json, '{}', {name: 'JSON Struktur mit den Daten vom Anrufbeantworter aus der FritzBox', unit: '', type: 'string', role: 'value', def: '{}'});
createState(DP_Fritzbox_AnrufbeantworterDatenAktualisieren, false,{name: 'Manueller Trigger um die Daten aus dem Anrufbeantworter auszulesen', unit: '', read: true, write: true, type: 'boolean', role: 'button', def: false}); 
createState(DP_Fritzbox_AnrufbeantworterGesamtAnzahlNachrichten, 0, {name: 'Gesamtanzahl der Nachrichten auf dem Anrufbeantworter', unit: '', type: 'number', role: 'value', def: 0});
createState(DP_Fritzbox_AnrufbeantworterAnzahlNeueNachrichten, 0, {name: 'Anzahl der neuen Nachrichten auf dem Anrufbeantworter', unit: '', type: 'number', role: 'value', def: 0});
createState(DP_Fritzbox_AnrufbeantworterDeleteMessage, '', {name: 'Zum loeschen ausgewaehlter Eintrag vom Anrufbeantworter', unit: '', read: true, write: true, type: 'string', role: 'value', def: ''});
createState(DP_Fritzbox_AnrufbeantworterIndexMessage_json, '', {name: 'JSON Struktur mit den Anrufbeantworter Index Eintraegen um sie in einem Select Widget darstellen zu koennen', unit: '', read: true, write: true, type: 'string', role: 'value', def: ''});

//Funktion Fritzbox_Anrufbeantworter_DeleteMessage() löscht die Nachricht welche als Index übergeben wird und liest danach erneut alle Anrufe vom Anrufbeantworter wieder aus
function Fritzbox_Anrufbeantworter_DeleteMessage(NewMessageIndex){
   var befehl_DeleteMessage = '{"service": "urn:dslforum-org:service:X_AVM-DE_TAM:1","action": "DeleteMessage","params": {"NewIndex": "'+ NewIndex_Anrufbeantworter + '", "NewMessageIndex": "' + NewMessageIndex + '" }}';
 
   setState("tr-064.1.states.command","{}");
   setState("tr-064.1.states.command",befehl_DeleteMessage); //Befehl zum loeschen einer Nachricht im Anrufbeantworter
   if(debug) console.log("Antwort auf command im State tr-064.1.states.commandResult: " + getState("tr-064.1.states.commandResult").val);
 
   Fritzbox_Anrufbeantworter_GetMessageList();
}

on({id:DP_Fritzbox_AnrufbeantworterDeleteMessage, change: 'any'}, function(obj) {
    if(debug) console.log("Es wird der Eintrag auf dem Anrufebeantworter mit dem Index: " + getState(DP_Fritzbox_AnrufbeantworterDeleteMessage).val + "gelöscht");
    Fritzbox_Anrufbeantworter_DeleteMessage(getState(DP_Fritzbox_AnrufbeantworterDeleteMessage).val);
});

var AB_Index_DeleteMessageIndex = {};
 
AB_Index_DeleteMessageIndex.sendTo = function(text, subText = '', value = '', icon = '', iconColor = '') {
   let json = getState(DP_Fritzbox_AnrufbeantworterIndexMessage_json).val;

   if (json) {
       try {
           json = JSON.parse(json);
       } catch (e) {
           json = [];
           console.warn('Wert ist kein JSON string! Wert wird ersetzt!');
       }
   } else {
       json = [];
   }
   json.push(
       {
           text: text,
           subText: subText,
           value: value,
           icon: icon,
           iconColor: iconColor,
       }
   )
   setState(DP_Fritzbox_AnrufbeantworterIndexMessage_json, JSON.stringify(json), true);
}
 
// Funktion Fritzbox_Anrufbeantworter_GetMessageList() liest aus der Fritzbox die hinterlegten Informationen zu den Anrufen auf dem 
// Anrufbeantworter aus. Ergebnis als JSON in einen Datenpunkt gespeichert, damit es in VIS einfach dargestellt werden kann. Es werden
// in Datenpunkten gespeichert  wieviele Anrufe im Anrufbeantworter insgesamt vorliegen und wieviele neue Nachrichten vorhanden sind

function Fritzbox_Anrufbeantworter_GetMessageList(){
   //Skript zum parsen von XML zu JSON: https://forum.iobroker.net/topic/623/gel%C3%B6st-xml-daten-einer-url-weiterverarbeiten/19
   //Damit das xml geparsed werden kann muss in der Javascript Instanz unter "Zusätzliche NPM Module" noch "xml2js" (mit Enter bestätigen) eintragen werden
 
   var parseString = require('xml2js').parseString;
   var request = require('request');
 
   var Result_Fritzbox_HyperlinkXmlTAM;
   var Fritzbox_AnrufbeantworterAnzahlNeueNachrichtenn = 0;
   var Fritzbox_AnrufbeantworterDaten_json = "";
   var befehl_GetMessageList = '{"service": "urn:dslforum-org:service:X_AVM-DE_TAM:1","action": "GetMessageList","params": {"NewIndex ": "' + NewIndex_Anrufbeantworter + '"}}';
 
   setState("tr-064.1.states.command","{}");
   setState("tr-064.1.states.command",befehl_GetMessageList); //Befehl zum auslesen der Anrufbeantworterdaten in Datenpunkt schreiben
   if(debug) console.log("Antwort auf command im State tr-064.1.states.commandResult: " + getState("tr-064.1.states.commandResult").val);
 
   setState(DP_Fritzbox_AnrufbeantworterIndexMessage_json,"");  //Setzt den aktuellen Inhalt vom Datenpunkt zurück, damit im Verlauf die Index Nummer von den Anrufen neu geschrieben werden können
 
   //Das Ergebnis im Datenpunkt commandResult ist ein Link auf ein XML welches die Informationen zu den Anrufen auf dem
   //Anrufbeantworter enthält. Das Ergebnis hat folgendes Format: {"NewURL":"http://192.168.178.1:49000/tamcalllist.lua?sid=2a4abe5e5ad61b64&tamindex=0"}
   //Aus diesem String wird mittels substring der eigentliche Link extrahiert
   Result_Fritzbox_HyperlinkXmlTAM = getState("tr-064.1.states.commandResult").val;
   Result_Fritzbox_HyperlinkXmlTAM = Result_Fritzbox_HyperlinkXmlTAM.substring(11, getState("tr-064.1.states.commandResult").val.length -2);  //die reine URL wird extrahiert
   if(debug) console.log("Extrahierter Hyperlink aus commandresult. CommandResult: " + getState("tr-064.1.states.commandResult").val + " und der extrahierte Link: " + Result_Fritzbox_HyperlinkXmlTAM);
 
   //Das XML File wird geparst und in eine JSON Struktur umgewandelt    
   request(Result_Fritzbox_HyperlinkXmlTAM, function (error, response, body) {
       if (!error && response.statusCode == 200) {
       if(debug) console.log("Body: " + body);
 
       //Aus dem xml String wird der Wert nach tam calls: extrahiert welcher die Anzahl der Anrufe auf dem Anrufbeantworter angibt
       //Beispiel: ... <!-- tam calls:0 --> </Root> ...
       //https://regex101.com/
       //https://regex101.com/r/Q74grJ/1
 
       const regex = /<!-- tam calls:[\s\S]*?(\d+)/igm;
       const matches = regex.exec(body);
       const FB_xml_TamCalls = parseInt(matches[1]);      
 
       if(debug) console.log("FB_xml_TamCalls aus XML extrahiert: "+ FB_xml_TamCalls);
       setState(DP_Fritzbox_AnrufbeantworterGesamtAnzahlNachrichten, FB_xml_TamCalls);
 
       parseString(body, {
               explicitArray: true, // Always put child nodes in an array if true; otherwise an array is created only if there is more than one.
               mergeAttrs: true //Merge attributes and child elements as properties of the parent, instead of keying attributes off a child attribute object. This option is ignored if ignoreAttrs is true.
           }, 
 
           function (err, result) {
               if (err) {
                   log("Fehler: " + err);
               } 
               else {
                   if(debug) console.log("Result Umwandlung xml in JSON: " + JSON.stringify(result));
 
                   //Abhaengig von der Anzahl der Anruf auf dem Anrufbeantworter erfolgt die weitere Verarbeitung der Daten
                   if(FB_xml_TamCalls == 0){       //Wenn keine Nachrichten auf dem Anrufbeantworter in der Fritzbox vorliegen
                       
                       //JSON String aus der Fritzbox wenn keine Nachricht auf dem AB ist
                       //JSON: {"Root":"\n\n\n\n"}
                       
                       Fritzbox_AnrufbeantworterDaten_json = '[{"Index":["---"],"Tam":["---"],"Called":["---"],"Date":["---"],"Duration":["---"],"Inbook":["---"],"Name":["---"],"New":["---"],"Number":["---"],"Path":["---"]}]';
 
                       setState(DP_Fritzbox_AnrufbeantworterAnzahlNeueNachrichten,0); //Anzahl der neuen Nachrichten auf 0 setzen
                       setState(DP_Fritzbox_AnrufbeantworterDaten_json,Fritzbox_AnrufbeantworterDaten_json);
 
                       setTimeout(function(){
                           AB_Index_DeleteMessageIndex.sendTo("Keine Nachricht vorhanden","","","phone-message-outline","red");
                       },500);
                   }
                   else{                           //Mindestens eine Nachricht ist auf dem Anrufbeantworter in der Fritzbox vorhanden
                       
                       //JSON String wenn eine Nachricht auf dem Anrufbeantworter ist
                       //JSON: {"Root":{"Message":[{"Index":["0"],"Tam":["0"],"Called":["0123456789"],"Date":["09.09.20 21:17"],"Duration":["0:01"],"Inbook":["0"],"Name":["Feuersturm"],"New":["1"],"Number":["0987654321"],"Path":["/download.lua?path=/data/tam/rec/rec.0.000"]}]}}
 
                       //JSON String wenn zwei Nachrichten auf dem Anrufbeantworter sind
                       //JSON: {"Root":{"Message":[{"Index":["1"],"Tam":["0"],"Called":["0123456789"],"Date":["09.09.20 21:30"],"Duration":["0:01"],"Inbook":["0"],"Name":["Feuersturm"],"New":["1"],"Number":["0987654321"],"Path":["/download.lua?path=/data/tam/rec/rec.0.001"]},{"Index":["0"],"Tam":["0"],"Called":["0123456789"],"Date":["09.09.20 21:17"],"Duration":["0:01"],"Inbook":["0"],"Name":["Feuersturm"],"New":["0"],"Number":["0987654321"],"Path":["/download.lua?path=/data/tam/rec/rec.0.000"]}]}}
 
                       Fritzbox_AnrufbeantworterDaten_json = JSON.stringify(result);
                       Fritzbox_AnrufbeantworterDaten_json = Fritzbox_AnrufbeantworterDaten_json.substring(19,JSON.stringify(result).length - 2);
                       setState(DP_Fritzbox_AnrufbeantworterDaten_json,Fritzbox_AnrufbeantworterDaten_json);
 
                       if(debug) console.log("result.Root.Message.length: " + JSON.stringify(result.Root.Message.length));
 
                       for (let i = 0; i < JSON.stringify(result.Root.Message.length); i++) {
 
                           let MessageIndexValue = JSON.stringify(result.Root.Message[i].Index);
                           MessageIndexValue = MessageIndexValue.substring(2,MessageIndexValue.length - 2);
 
                           if(debug) console.log("Nachricht neu [i]: [" + i + "]" +  JSON.stringify(result.Root.Message[i].New) + " mit Index: " + JSON.stringify(result.Root.Message[i].Index) + " und gekürzt: " + MessageIndexValue);
                           
                           setTimeout(function(){
                               AB_Index_DeleteMessageIndex.sendTo("AB Index","",MessageIndexValue,"phone-message-outline","red");
                           },i*500);
 
                           if( JSON.stringify(result.Root.Message[i].New) === '["1"]') {
                               Fritzbox_AnrufbeantworterAnzahlNeueNachrichtenn = Fritzbox_AnrufbeantworterAnzahlNeueNachrichtenn + 1; 
                           }
                       }
                       if(debug) console.log("Anzahl Neuer Nachrichten auf dem AB: " + Fritzbox_AnrufbeantworterAnzahlNeueNachrichtenn);
                       setState(DP_Fritzbox_AnrufbeantworterAnzahlNeueNachrichten,Fritzbox_AnrufbeantworterAnzahlNeueNachrichtenn);
                   }
               }
           });
   } 
   else  {
       log(error);
   }
   });
}
 
//Über Datenpunkt kann manuell das Auslesen der Daten aus dem Anrufbeantworter getriggert werden
on({id:DP_Fritzbox_AnrufbeantworterDatenAktualisieren, change: 'any'}, function(obj) {
        Fritzbox_Anrufbeantworter_GetMessageList(); //Es werden die Informationen aus dem Anrufbeantworter in der Fritzbox ausgelesen
});
 
//------------------------- Anrufername oder Telefonnummer ermitteln -----------
//Quelle: https://forum.iobroker.net/topic/15533/tr-64-erkennen-ob-ein-aufruf-auf-ab-gesprochen-wurde/5
/*
Steht der Anrufer in Adressbuch wird Vorname und Name ausgegeben, sonst die 
Telefonnummer
{1}
*/
 
function telefonname () {
    var tokens     =  getState("tr-064.1.callmonitor.inbound.callerName").val.split(",");
    var answer     = '';
    var vorname    =  tokens[1];
    var famname    =  tokens[0];
 
    if (famname === undefined) famname = '';
    if (vorname === undefined) vorname = '';
    if (vorname !== '' || famname !== '')
        answer =  vorname + ' ' + famname;
    else
        answer = getState("tr-064.1.callmonitor.inbound.caller").val;
    return answer; 
}
 
/*
---------------------------- Anruf beendet -------------------------------------
Quelle: https://forum.iobroker.net/topic/15533/tr-64-erkennen-ob-ein-aufruf-auf-ab-gesprochen-wurde/5
Es werden die Anzahl der Anrufe auf Anrufbeantworter und eine Liste von 
Namen mitgeführt, die Nachrichten hinterlassen haben.
Es werden dazu 'callmonitor.lastCall.type' und 
'tr-064.1.callmonitor.lastCall.extension' ausgewertet.
{1}
Mangels Dok wurde durch Ausprobieren ermittelt, dass
{1}
- type === missed                        : es wurde aufgelegt und nicht gesprochen.
- type === disconnect && extension !== 40: Gespräch wurde geführt und dann aufgelegt
- type === disconnect && extension === 40: Nachricht auf AB
{1}
bedeuten könnte.
*/
 
 
on({id:"tr-064.1.callmonitor.toPauseState", change: 'ne'}, function(obj) {
   setTimeout( function() {
   if (getState('tr-064.1.callmonitor.toPauseState').val === 'end') {
       var name = telefonname();
       if (getState("tr-064.1.callmonitor.lastCall.type").val === 'missed') {
           log(name + " hat aufgelegt und keine Nachricht hinterlassen");}
       if (getState("tr-064.1.callmonitor.lastCall.type").val === 'disconnect') {
           if (getState('tr-064.1.callmonitor.lastCall.extension').val == 40) {
                if(debug) console.log(name + " hat auf den Anrufbeantworter gesprochen. Daten werden aus der Fritzbox ausgelesen...");
                Fritzbox_Anrufbeantworter_GetMessageList(); //Es werden die Informationen aus dem Anrufbeantworter in der Fritzbox ausgelesen
           } else {
              log("Der Anruf von " + telefonname() + " hat " + getState("tr-064.1.callmonitor.lastCall.duration").val + " sec gedauert");
           }    
       }
   }    
   }, 3000);
});

// Alle 10 Minuten Anzeige aktualisieren
schedule('*/10 * * * *', Fritzbox_Anrufbeantworter_GetMessageList);